@@ -1,11 +1,14 @@
 from __future__ import annotations
-import dataclasses
-import emlib.misc
-from typing import TYPE_CHECKING
 
-if TYPE_CHECKING:
+import dataclasses
+
+
+import emlib.envir
+from . import synth as _synth
+
+import typing as _t
+if _t.TYPE_CHECKING:
     from . import engine
-    import ipywidgets as ipy
 
 
 def _guessStep(minval: float, maxval: float):
@@ -33,12 +36,15 @@ def _guessRange(value: float, namehint: str = '') -> tuple[float, float]:
             return (0, 12000)
         if ("midi" in namehint or "pitch" in namehint) and 0 <= value <= 127:
             return (0, 127)
+        if namehint == 'pan' or namehint == 'kpan' and 0 <= value <= 1:
+            return (0., 1.)
+
     if -90 < value < 0:
         return (-100, 0)
     if value < 0:
         val0, val1 = _guessRange(-value)
         return -val1, -val0
-    if value < 1:
+    if value <= 1:
         return (0, 2)
     elif value < 5:
         return (0, 10)
@@ -52,14 +58,29 @@ def _guessRange(value: float, namehint: str = '') -> tuple[float, float]:
 
 @dataclasses.dataclass
 class ParamSpec:
-    descr: str
-    minvalue: float
-    maxvalue: float
-    step: float = -1
-    startvalue: Optional[float] = None
-    widgetHint: str = 'slider'
+    name: str
+    """The parameter name"""
 
-    def __post_init__(self):
+    minvalue: float = 0.
+    maxvalue: float = 1.
+    step: float = -1
+    startvalue: float = 0.
+    widgetHint: str = 'slider'
+    valuescale: str = 'linear'
+
+    description: str = ''
+    """A optional description"""
+
+    def scale(self, x: float) -> float:
+        assert 0 <= x <= 1
+        if self.valuescale == 'linear':
+            return (self.maxvalue - self.minvalue) * x + self.minvalue
+        elif self.valuescale == 'log10':
+            return (self.maxvalue - self.minvalue) * (10**(1-x)) + self.minvalue
+        else:
+            raise ValueError(f"Invalid valuescale: {self.valuescale}")
+
+def __post_init__(self):
         if self.step == -1:
             self.step = _guessStep(self.minvalue, self.maxvalue)
 
@@ -76,8 +97,9 @@ def _stepToFormat(step: float) -> str:
     return fmt
 
 
-def _jupySlider(name:str, startvalue: float, minvalue: float, maxvalue: float,
-                callback:Callable, step:float=None, width='80%', log=False):
+def _jupyterSlider(name: str, startvalue: float, minvalue: float, maxvalue: float,
+                   callback: _t.Callable, step: float | None = None, width='80%',
+                   log=False):
     import ipywidgets as ipy
     if step is None:
         step = _guessStep(minvalue, maxvalue)
@@ -92,23 +114,21 @@ def _jupySlider(name:str, startvalue: float, minvalue: float, maxvalue: float,
         s = ipy.FloatLogSlider(value=startvalue, min=minvalue, max=maxvalue,
                                step=step, description=name, layout=layout,
                                readout_format=fmt)
-    if callback:
-        s.observe(lambda change:callback(change['new']), names='value')
+    s.observe(lambda change: callback(change['new']), names='value')
     return s
 
 
-def _jupyEntry(name: str, startvalue: float, minvalue:float, maxvalue: float,
-               callback: Callable):
+def _jupyterEntry(name: str, startvalue: float, minvalue:float, maxvalue: float,
+                  callback: _t.Callable):
     import ipywidgets as ipy
     step = 0.001
     w = ipy.BoundedFloatText(value=startvalue, min=minvalue, max=maxvalue,
                              step=step, description=name)
-    if callback:
-        w.observe(lambda change:print(change), names='value')
+    w.observe(lambda change: callback(change['new']), names='value')
     return w
 
 
-def interact(**sliders: dict[str, tuple[float, float, float, Callable]]):
+def interact(**sliders: tuple[float, float, float, _t.Callable]):
     """
     Creates a set of interactive widgets
 
@@ -119,7 +139,7 @@ def interact(**sliders: dict[str, tuple[float, float, float, Callable]]):
             with the current value of the widget
 
     Example
-    =======
+    ~~~~~~~
 
         from csoundengine import *
         from csoundengine.interact import *
@@ -135,24 +155,97 @@ def interact(**sliders: dict[str, tuple[float, float, float, Callable]]):
         endin
         ''')
         p1 = e.sched("foo", [0.1, 67])
-        sliders = makeSliders(kamp=(e.getp(p1, 4, 0, 1, lambda x:e.setp(p1, 4, x),
-                              kmidi=(e.getp(p1, 5, 0, 127, lambda x:e.setp(p1, 5, x))
-        display(*sliders)
+        interact(kamp=(0.1, 0., 1., lambda x:e.setp(p1, 4, x),
+                 kmidi=(67, 0, 127, lambda x:e.setp(p1, 5, x))
     """
     from IPython.display import display
     widgets = []
     for key, value in sliders.items():
         curvalue, minvalue, maxvalue, func = value
-        s = _jupySlider(name=key, startvalue=curvalue, minvalue=minvalue,
-                        maxvalue=maxvalue, callback=func)
+        s = _jupyterSlider(name=key, startvalue=curvalue, minvalue=minvalue,
+                           maxvalue=maxvalue, callback=func)
         widgets.append(s)
     display(*widgets)
 
 
-def interactPargs(engine: engine.Engine, 
+def guessParamSpecs(params: dict[str, float | int | str | None],
+                    ranges: dict[str, tuple[float, float]] = None
+                    ) -> list[ParamSpec]:
+    paramspecs: list[ParamSpec] = []
+    for paramname, value in params.items():
+        if isinstance(value, str):
+            continue
+        elif value is None:
+            value = 0.
+        if ranges and paramname in ranges:
+            minval, maxval = ranges[paramname]
+        else:
+            minval, maxval = _guessRange(value, paramname)
+        paramspecs.append(ParamSpec(name=paramname,
+                                    minvalue=minval,
+                                    maxvalue=maxval,
+                                    startvalue=value))
+    return paramspecs
+
+
+def interactSynth(synth: _synth.Synth | _synth.SynthGroup,
+                  specs: list[ParamSpec] = None) -> None:
+    """
+    Interact with a Synth
+
+    Args:
+        synth: the synth for which to generate a UI
+        specs: a list of ParamSpec
+    """
+    if not specs:
+        dynparams = synth.dynamicParamNames(aliases=False)
+        params = {param: synth.paramValue(param) for param in sorted(dynparams)}
+        specs = guessParamSpecs(params=params)
+
+    if emlib.envir.inside_jupyter():
+        return _interactSynthJupyter(synth=synth, specs=specs)
+    else:
+        raise RuntimeError("interact is only supported inside a jupyter session at the"
+                           " moment.")
+
+
+def _interactSynthJupyter(synth: _synth.Synth | _synth.SynthGroup,
+                          specs: list[ParamSpec],
+                          stopbutton=True,
+                          width='80%'
+                          ) -> None:
+    import ipywidgets as ipy
+    from IPython.display import display
+    widgets = []
+    if stopbutton:
+        button = ipy.Button(description="Stop")
+        button.on_click(lambda *args, s=synth: s.stop())
+        widgets.append(button)
+
+    for spec in specs:
+        if spec.widgetHint == 'slider':
+            w = _jupyterSlider(name=spec.name,
+                               startvalue=spec.startvalue,
+                               minvalue=spec.minvalue,
+                               maxvalue=spec.maxvalue,
+                               width=width,
+                               callback=lambda val, s=synth, p=spec.name: s.set(p, value=val))
+        elif spec.widgetHint == 'entry':
+            w = _jupyterEntry(name=spec.name,
+                              startvalue=spec.startvalue,
+                              minvalue=spec.minvalue,
+                              maxvalue=spec.maxvalue,
+                              callback=lambda val, s=synth, p=spec.name: s.set(p, value=val))
+        else:
+            raise ValueError(f"Widget hint not understood: {spec.widgetHint}")
+        widgets.append(w)
+    display(*widgets)
+
+
+def interactPargs(engine: engine.Engine,
                   p1: float | str,
-                  specs: dict[int|str, ParamSpec]=None,
-                  **namedSpecs):
+                  specs: dict[int|str, ParamSpec] = {},
+                  **namedSpecs: ParamSpec):
     """
     Interact with pfields of an event
 
@@ -189,7 +282,7 @@ def interactPargs(engine: engine.Engine,
         allspecs.update(specs)
     if namedSpecs:
         allspecs.update(namedSpecs)
-    if emlib.misc.inside_jupyter():
+    if emlib.envir.inside_jupyter():
         return _jupyInteractPargs(engine=engine, p1=p1, specs=allspecs)
     else:
         raise RuntimeError("interact is only supporte inside a jupyter session at the"
@@ -198,7 +291,7 @@ def interactPargs(engine: engine.Engine,
 
 def _jupyInteractPargs(engine: engine.Engine,
                        p1: float|str,
-                       specs: dict[int|str, ParamSpec] = None,
+                       specs: dict[int|str, ParamSpec],
                        stopbutton=True,
                        width='80%'):
     """
@@ -226,8 +319,8 @@ def _jupyInteractPargs(engine: engine.Engine,
         event = e.sched(100, args=[0.1, 67])
         e.interact(event, p4=ParamSpec('kamp', 0, 1){4: ParamSpec('kamp', 0, 1)})
     """
-    from IPython.display import display
     import ipywidgets as ipy
+    from IPython.display import display
     widgets = []
     if stopbutton:
         button = ipy.Button(description="Stop")
@@ -236,18 +329,22 @@ def _jupyInteractPargs(engine: engine.Engine,
 
     for key, spec in specs.items():
         idx = key if isinstance(key, int) else int(key[1:])
-        value0 = engine.getp(p1,idx) if spec.startvalue is None else spec.startvalue
+        if isinstance(p1, str):
+            p1 = engine.queryNamedInstr(p1)
+        currentvalue = engine._getp(p1, idx)
+        value0 = currentvalue if currentvalue is not None else spec.startvalue
+        if value0 is None:
+            value0 = spec.minvalue
         if spec.widgetHint == 'slider':
-            w = _jupySlider(spec.descr, startvalue=value0,
-                            minvalue=spec.minvalue, maxvalue=spec.maxvalue,
-                            width=width,
-                            callback=lambda val, p1=p1, idx=idx: engine.setp(p1, idx, val))
+            w = _jupyterSlider(spec.description, startvalue=value0,
+                               minvalue=spec.minvalue, maxvalue=spec.maxvalue,
+                               width=width,
+                               callback=lambda val, p1=p1, idx=idx: engine.setp(p1, idx, val))
         elif spec.widgetHint == 'entry':
-            w = _jupyEntry(spec.descr, startvalue=value0,
-                           minvalue=spec.minvalue, maxvalue=spec.maxvalue,
-                           callback=lambda val, p1=p1, idx=idx: engine.setp(p1, idx, val))
+            w = _jupyterEntry(spec.description, startvalue=value0,
+                              minvalue=spec.minvalue, maxvalue=spec.maxvalue,
+                              callback=lambda val, p1=p1, idx=idx: engine.setp(p1, idx, val))
         else:
             raise ValueError(f"Widget hint not understood: {spec.widgetHint}")
         widgets.append(w)
     display(*widgets)
-
